@@ -1,10 +1,10 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 #****************************************************************************
 # convertdlg.py, provides the main dialog and GUI interface
 #
 # ConvertAll, a units conversion program
-# Copyright (C) 2010, Douglas W. Bell
+# Copyright (C) 2014, Douglas W. Bell
 #
 # This is free software; you can redistribute it and/or modify it under the
 # terms of the GNU General Public License, either Version 2 or any later
@@ -17,13 +17,12 @@ import os.path
 from PyQt4 import QtCore, QtGui
 try:
     from __main__ import __version__, __author__, helpFilePath, iconPath
-    from __main__ import lang, localEncoding
+    from __main__ import lang
 except ImportError:
     __version__ = __author__ = '??'
     helpFilePath = None
     iconPath = None
     lang = ''
-    localEncoding = 'utf-8'
 import unitdata
 from unitgroup import UnitGroup
 from option import Option
@@ -40,7 +39,8 @@ import optiondlg
 
 
 class ConvertDlg(QtGui.QWidget):
-    """Main dialog for ConvertAll program"""
+    """Main dialog for ConvertAll program.
+    """
     unitData = unitdata.UnitData()
     def __init__(self, parent=None):
         QtGui.QWidget.__init__(self, parent)
@@ -51,7 +51,7 @@ class ConvertDlg(QtGui.QWidget):
         iconPathList = [iconPath, os.path.join(modPath, 'icons/'),
                          os.path.join(modPath, '../icons')]
         self.icons = icondict.IconDict()
-        self.icons.addIconPath(filter(None, iconPathList))
+        self.icons.addIconPath([path for path in iconPathList if path])
         try:
             QtGui.QApplication.setWindowIcon(self.icons['convertall_med'])
         except KeyError:
@@ -63,14 +63,12 @@ class ConvertDlg(QtGui.QWidget):
         self.recentUnits = recentunits.RecentUnits(self.option)
         try:
             num = ConvertDlg.unitData.readData()
-        except unitdata.UnitDataError, text:
+        except unitdata.UnitDataError as text:
             QtGui.QMessageBox.warning(self, 'ConvertAll',
-                                      _('Error in unit data - %s') % text)
+                                      _('Error in unit data - {0}').
+                                      format(text))
             sys.exit(1)
-        try:
-            print (_('%d units loaded') % num).encode(localEncoding)
-        except UnicodeError:
-            print (_('%d units loaded') % num).encode('utf-8')
+        print(_('{0} units loaded').format(num))
         self.fromGroup = UnitGroup(ConvertDlg.unitData, self.option)
         self.toGroup = UnitGroup(ConvertDlg.unitData, self.option)
         self.origPal = QtGui.QApplication.palette()
@@ -93,10 +91,9 @@ class ConvertDlg(QtGui.QWidget):
         self.fromUnitEdit.setFocus()
         self.fromUnitListView = unitlistview.UnitListView(self.fromGroup, 0)
         fromLayout.addWidget(self.fromUnitListView)
-        self.connect(self.fromUnitEdit, QtCore.SIGNAL('currentChanged'),
-                     self.fromUnitListView.updateSelection)
-        self.connect(self.fromUnitListView, QtCore.SIGNAL('unitChanged'),
-                     self.fromUnitEdit.unitUpdate)
+        self.fromUnitEdit.currentChanged.connect(self.fromUnitListView.
+                                                 updateSelection)
+        self.fromUnitListView.unitChanged.connect(self.fromUnitEdit.unitUpdate)
         self.fromUnitListView.setFocusProxy(self.fromUnitEdit)
         self.addButtons(self.fromGroup, self.fromUnitListView, fromLayout)
 
@@ -108,10 +105,9 @@ class ConvertDlg(QtGui.QWidget):
         toLayout.addWidget(self.toUnitEdit)
         self.toUnitListView = unitlistview.UnitListView(self.toGroup, 1)
         toLayout.addWidget(self.toUnitListView)
-        self.connect(self.toUnitEdit, QtCore.SIGNAL('currentChanged'),
-                     self.toUnitListView.updateSelection)
-        self.connect(self.toUnitListView, QtCore.SIGNAL('unitChanged'),
-                     self.toUnitEdit.unitUpdate)
+        self.toUnitEdit.currentChanged.connect(self.toUnitListView.
+                                               updateSelection)
+        self.toUnitListView.unitChanged.connect(self.toUnitEdit.unitUpdate)
         self.toUnitListView.setFocusProxy(self.toUnitEdit)
         self.addButtons(self.toGroup, self.toUnitListView, toLayout)
         self.showHideButtons()
@@ -129,8 +125,7 @@ class ConvertDlg(QtGui.QWidget):
                                            fromNumBox, statusLabel,
                                            self.recentUnits, True)
         fromNumLayout.addWidget(self.fromNumEdit)
-        self.connect(self.fromUnitEdit, QtCore.SIGNAL('unitChanged'),
-                     self.fromNumEdit.unitUpdate)
+        self.fromUnitEdit.unitChanged.connect(self.fromNumEdit.unitUpdate)
         self.fromNumEdit.setEnabled(False)
         equalsLabel = QtGui.QLabel(' = ')
         equalsLabel.setFont(QtGui.QFont(self.font().family(), 30))
@@ -143,45 +138,37 @@ class ConvertDlg(QtGui.QWidget):
                                          toNumBox, statusLabel,
                                          self.recentUnits, False)
         toNumLayout.addWidget(self.toNumEdit)
-        self.connect(self.toUnitEdit, QtCore.SIGNAL('unitChanged'),
-                     self.toNumEdit.unitUpdate)
+        self.toUnitEdit.unitChanged.connect(self.toNumEdit.unitUpdate)
         self.toNumEdit.setEnabled(False)
-        self.connect(self.fromNumEdit, QtCore.SIGNAL('convertNum'),
-                     self.toNumEdit.setNum)
-        self.connect(self.toNumEdit, QtCore.SIGNAL('convertNum'),
-                     self.fromNumEdit.setNum)
-        self.connect(self.fromNumEdit, QtCore.SIGNAL('convertNum'),
-                     self.setRecentAvail)
-        self.connect(self.toNumEdit, QtCore.SIGNAL('convertNum'),
-                     self.setRecentAvail)
-        self.connect(self.fromNumEdit, QtCore.SIGNAL('convertRqd'),
-                     self.toNumEdit.convert)
-        self.connect(self.toNumEdit, QtCore.SIGNAL('convertRqd'),
-                     self.fromNumEdit.convert)
+        self.fromNumEdit.convertNum.connect(self.toNumEdit.setNum)
+        self.toNumEdit.convertNum.connect(self.fromNumEdit.setNum)
+        self.fromNumEdit.convertNum.connect(self.setRecentAvail)
+        self.toNumEdit.convertNum.connect(self.setRecentAvail)
+        self.fromNumEdit.convertRqd.connect(self.toNumEdit.convert)
+        self.toNumEdit.convertRqd.connect(self.fromNumEdit.convert)
 
         buttonLayout = QtGui.QVBoxLayout()     # major buttons
         topLayout.addLayout(buttonLayout)
         closeButton = QtGui.QPushButton(_('&Close'))
         buttonLayout.addWidget(closeButton)
         closeButton.setFocusPolicy(QtCore.Qt.NoFocus)
-        self.connect(closeButton, QtCore.SIGNAL('clicked()'), self.close)
+        closeButton.clicked.connect(self.close)
         finderButton = QtGui.QPushButton(_('&Unit Finder...'))
         buttonLayout.addWidget(finderButton)
         finderButton.setFocusPolicy(QtCore.Qt.NoFocus)
-        self.connect(finderButton, QtCore.SIGNAL('clicked()'), self.showFinder)
+        finderButton.clicked.connect(self.showFinder)
         optionsButton = QtGui.QPushButton(_('&Options...'))
         buttonLayout.addWidget(optionsButton)
         optionsButton.setFocusPolicy(QtCore.Qt.NoFocus)
-        self.connect(optionsButton, QtCore.SIGNAL('clicked()'),
-                     self.changeOptions)
+        optionsButton.clicked.connect(self.changeOptions)
         helpButton = QtGui.QPushButton(_('&Help...'))
         buttonLayout.addWidget(helpButton)
         helpButton.setFocusPolicy(QtCore.Qt.NoFocus)
-        self.connect(helpButton, QtCore.SIGNAL('clicked()'), self.help)
+        helpButton.clicked.connect(self.help)
         aboutButton = QtGui.QPushButton(_('&About...'))
         buttonLayout.addWidget(aboutButton)
         aboutButton.setFocusPolicy(QtCore.Qt.NoFocus)
-        self.connect(aboutButton, QtCore.SIGNAL('clicked()'), self.about)
+        aboutButton.clicked.connect(self.about)
         buttonLayout.addStretch()
 
         xSize = self.option.intData('MainDlgXSize', 0, 10000)
@@ -190,9 +177,23 @@ class ConvertDlg(QtGui.QWidget):
             self.resize(xSize, ySize)
         self.move(self.option.intData('MainDlgXPos', 0, 10000),
                   self.option.intData('MainDlgYPos', 0, 10000))
+        if self.option.boolData('LoadLastUnit') and len(self.recentUnits) > 1:
+            self.fromGroup.update(self.recentUnits[0])
+            self.fromUnitEdit.unitUpdate()
+            self.fromUnitListView.updateSelection()
+            self.toGroup.update(self.recentUnits[1])
+            self.toUnitEdit.unitUpdate()
+            self.toUnitListView.updateSelection()
+            self.fromNumEdit.setFocus()
+            self.fromNumEdit.selectAll()
+        if self.option.boolData('ShowStartupTip'):
+            self.show()
+            tipDialog = TipDialog(self.option, self)
+            tipDialog.exec_()
 
     def addButtons(self, unitGroup, listView, upperLayout):
-        """Add buttons to unit selector"""
+        """Add buttons to unit selector.
+        """
         buttonLayout = QtGui.QHBoxLayout()
         upperLayout.addLayout(buttonLayout)
         buttons = []
@@ -208,38 +209,40 @@ class ConvertDlg(QtGui.QWidget):
         upperLayout.addLayout(extraLayout)
         extraLayout.addWidget(buttons[-1])
         for but in buttons:
-            self.connect(but, QtCore.SIGNAL('stateChg'), listView.relayChange)
+            but.stateChg.connect(listView.relayChange)
             but.setEnabled(False)
             self.textButtons.append(but)
         buttons[-1].setEnabled(True)
         recentButton = QtGui.QPushButton(_('Recent Unit'))
         recentButton.setFocusPolicy(QtCore.Qt.NoFocus)
         recentButton.unitGroup = unitGroup
-        self.connect(recentButton, QtCore.SIGNAL('clicked()'), self.recentMenu)
+        recentButton.clicked.connect(self.recentMenu)
         extraLayout.addWidget(recentButton)
         self.textButtons.append(recentButton)
         self.recentButtons.append(recentButton)
         self.setRecentAvail()
 
     def recentMenu(self):
-        """Show a menu with recently used units"""
+        """Show a menu with recently used units.
+        """
         button = self.sender()
         menu = QtGui.QMenu()
         for unit in self.recentUnits:
             action = menu.addAction(unit)
             action.unitGroup = button.unitGroup
-        self.connect(menu, QtCore.SIGNAL('triggered(QAction*)'),
-                     self.insertRecent)
+            menu.triggered.connect(self.insertRecent)
         menu.exec_(button.mapToGlobal(QtCore.QPoint(0, 0)))
 
     def setRecentAvail(self):
-        """Enable or disable recent unit button"""
+        """Enable or disable recent unit button.
+        """
         for button in self.recentButtons:
             button.setEnabled(len(self.recentUnits))
 
     def insertRecent(self, action):
-        """Insert the recent unit from the given action"""
-        action.unitGroup.update(unicode(action.text()))
+        """Insert the recent unit from the given action.
+        """
+        action.unitGroup.update(action.text())
         if action.unitGroup is self.fromGroup:
             self.fromUnitEdit.unitUpdate()
             self.fromUnitListView.updateSelection()
@@ -248,7 +251,8 @@ class ConvertDlg(QtGui.QWidget):
             self.toUnitListView.updateSelection()
 
     def updateColors(self):
-        """Adjust the colors to the current option settings"""
+        """Adjust the colors to the current option settings.
+        """
         if self.option.boolData('UseDefaultColors'):
             pal = self.origPal
         else:
@@ -260,13 +264,15 @@ class ConvertDlg(QtGui.QWidget):
         QtGui.QApplication.setPalette(pal)
 
     def showFinder(self):
-        """Show dialog for searhing and filtering units"""
+        """Show dialog for searhing and filtering units.
+        """
         if not self.findDlg:
             self.findDlg = finddlg.FindDlg(self)
         self.findDlg.show()
 
     def changeOptions(self):
-        """Show dialog for option changes"""
+        """Show dialog for option changes.
+        """
         origBackground = self.getOptionColor('Background')
         origForeground = self.getOptionColor('Foreground')
         dlg = optiondlg.OptionDlg(self.option, self)
@@ -278,10 +284,14 @@ class ConvertDlg(QtGui.QWidget):
         optiondlg.OptionDlgBool(dlg, 'FixedDecimals',
                                 _('Use fixed decimal places'))
         dlg.startGroupBox(_('Recent Units'))
-        optiondlg.OptionDlgInt(dlg, 'RecentUnits', _('Number saved'), 0, 99)
-        dlg.startGroupBox(_('Buttons'))
+        optiondlg.OptionDlgInt(dlg, 'RecentUnits', _('Number saved'), 2, 99)
+        optiondlg.OptionDlgBool(dlg, 'LoadLastUnit',
+                                _('Load last units at startup'))
+        dlg.startGroupBox(_('User Interface'))
         optiondlg.OptionDlgBool(dlg, 'ShowOpButtons',
                                 _('Show operator buttons'))
+        optiondlg.OptionDlgBool(dlg, 'ShowStartupTip',
+                                _('Show tip at startup'))
         dlg.startGroupBox(_('Colors'))
         optiondlg.OptionDlgBool(dlg, 'UseDefaultColors',
                                 _('Use default system colors'))
@@ -299,33 +309,38 @@ class ConvertDlg(QtGui.QWidget):
             self.setOptionColor('Foreground', origForeground)
 
     def getOptionColor(self, rootName):
-        """Return a color from option storage"""
-        return QtGui.QColor(self.option.intData('%sR' % rootName, 0, 255),
-                            self.option.intData('%sG' % rootName, 0, 255),
-                            self.option.intData('%sB' % rootName, 0, 255))
+        """Return a color from option storage.
+        """
+        return QtGui.QColor(self.option.intData(rootName + 'R', 0, 255),
+                            self.option.intData(rootName + 'G', 0, 255),
+                            self.option.intData(rootName + 'B', 0, 255))
 
     def setOptionColor(self, rootName, color):
-        """Store given color in options"""
-        self.option.changeData('%sR' % rootName, str(color.red()), True)
-        self.option.changeData('%sG' % rootName, str(color.green()), True)
-        self.option.changeData('%sB' % rootName, str(color.blue()), True)
+        """Store given color in options.
+        """
+        self.option.changeData(rootName + 'R', repr(color.red()), True)
+        self.option.changeData(rootName + 'G', repr(color.green()), True)
+        self.option.changeData(rootName + 'B', repr(color.blue()), True)
 
     def backColor(self):
-        """Allow user to set control background color"""
+        """Allow user to set control background color.
+        """
         background = self.getOptionColor('Background')
         newColor = QtGui.QColorDialog.getColor(background, self)
         if newColor.isValid() and newColor != background:
             self.setOptionColor('Background', newColor)
 
     def textColor(self):
-        """Allow user to set control text color"""
+        """Allow user to set control text color.
+        """
         foreground = self.getOptionColor('Foreground')
         newColor = QtGui.QColorDialog.getColor(foreground, self)
         if newColor.isValid() and newColor != foreground:
             self.setOptionColor('Foreground', newColor)
 
     def showHideButtons(self):
-        """Show or hide text modify buttons"""
+        """Show or hide text modify buttons.
+        """
         visible = self.option.boolData('ShowOpButtons')
         for button in self.textButtons:
             if visible:
@@ -334,7 +349,8 @@ class ConvertDlg(QtGui.QWidget):
                 button.hide()
 
     def findHelpFile(self):
-        """Return the path to the help file"""
+        """Return the path to the help file.
+        """
         modPath = os.path.abspath(sys.path[0])
         if modPath.endswith('.zip'):  # for py2exe
             modPath = os.path.dirname(modPath)
@@ -342,9 +358,9 @@ class ConvertDlg(QtGui.QWidget):
                     modPath, os.path.join(modPath, 'doc/')]
         fileList = ['README.html']
         if lang and lang != 'C':
-            fileList[0:0] = ['README_%s.html' % lang,
-                             'README_%s.html' % lang[:2]]
-        for path in filter(None, pathList):
+            fileList[0:0] = ['README_{0}.html'.format(lang),
+                             'README_{0}.html'.format(lang[:2])]
+        for path in [path for path in pathList if path]:
             for fileName in fileList:
                 fullPath = os.path.join(path, fileName)
                 if os.access(fullPath, os.R_OK):
@@ -352,25 +368,29 @@ class ConvertDlg(QtGui.QWidget):
         return ''
 
     def help(self):
-        """View the ReadMe file"""
+        """View the ReadMe file.
+        """
         if not self.helpView:
             path = self.findHelpFile()
             if not path:
                 QtGui.QMessageBox.warning(self, 'ConvertAll',
                                           _('Read Me file not found'))
                 return
-            self.helpView = helpview.HelpView(path, _('ConvertAll README File'),
+            self.helpView = helpview.HelpView(path,
+                                              _('ConvertAll README File'),
                                               self.icons)
         self.helpView.show()
 
     def about(self):
-        """Show about info"""
+        """Show about info.
+        """
         QtGui.QMessageBox.about(self, 'ConvertAll',
-                                _('ConvertAll Version %s\nby %s') %
-                                (__version__, __author__))
+                                _('ConvertAll Version {0}\nby {1}').
+                                format(__version__, __author__))
 
     def closeEvent(self, event):
-        """Save window data on close"""
+        """Save window data on close.
+        """
         self.option.changeData('MainDlgXSize', self.width(), True)
         self.option.changeData('MainDlgYSize', self.height(), True)
         self.option.changeData('MainDlgXPos', self.x(), True)
@@ -383,3 +403,49 @@ class ConvertDlg(QtGui.QWidget):
         self.recentUnits.writeList()
         self.option.writeChanges()
         event.accept()
+
+
+class TipDialog(QtGui.QDialog):
+    """Show a static usage tip at startup by default.
+    """
+    def __init__(self, option, parent=None):
+        QtGui.QDialog.__init__(self, parent)
+        self.option = option
+        self.setWindowFlags(QtCore.Qt.Dialog | QtCore.Qt.WindowTitleHint |
+                            QtCore.Qt.WindowSystemMenuHint)
+        self.setWindowTitle(_('Convertall - Tip'))
+        topLayout = QtGui.QVBoxLayout(self)
+        self.setLayout(topLayout)
+
+        box = QtGui.QGroupBox(_('Combining Units'))
+        topLayout.addWidget(box)
+        boxLayout = QtGui.QVBoxLayout(box)
+        label = QtGui.QLabel(self)
+        label.setTextFormat(QtCore.Qt.RichText)
+        label.setText(_('<p>ConvertAll\'s strength is the ability to combine '
+                        'units:</p>'
+                        '<ul><li>Enter "m/s" to get meters per second</li>'
+                        '<li>Enter "ft*lbf" to get foot-pounds (torque)</li>'
+                        '<li>Enter "in^2" to get square inches</li>'
+                        '<li>Enter "m^3" to get cubic meters</li>'
+                        '<li>or any other combinations you can imagine</li>'
+                        '</ul>'))
+        boxLayout.addWidget(label)
+
+        ctrlLayout = QtGui.QHBoxLayout()
+        topLayout.addLayout(ctrlLayout)
+        self.showCheck = QtGui.QCheckBox(_('Show this tip at startup'), self)
+        self.showCheck.setChecked(True)
+        ctrlLayout.addWidget(self.showCheck)
+
+        ctrlLayout.addStretch()
+        okButton = QtGui.QPushButton(_('&OK'), self)
+        ctrlLayout.addWidget(okButton)
+        okButton.clicked.connect(self.accept)
+
+    def accept(self):
+        """Called by dialog when OK button pressed.
+        """
+        if not self.showCheck.isChecked():
+            self.option.changeData('ShowStartupTip', 'no', True)
+        QtGui.QDialog.accept(self)
